@@ -40,6 +40,9 @@ svydesign.character<-function (ids, probs = NULL, strata = NULL, variables = NUL
 print.DBIsvydesign<-function(x,...){
   cat("DB-backed ")
   NextMethod()
+   if (!checkConnection(x$db$connection, error=FALSE))
+    cat("<DBI Connection closed>\n")
+  invisible(x)
 }
 
 summary.DBIsvydesign<-function(object,...){
@@ -64,18 +67,18 @@ open.DBIsvydesign<-function(con,...){
 }
 
 svymean.DBIsvydesign<-function(x, design,...){
-  design$variables<-getvars(x, design$db$connection, design$db$tablename,updates=design$updates)
+  design$variables<-getvars(x, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset)
   NextMethod("svymean",design)
 }
 
 
 svytotal.DBIsvydesign<-function(x, design,na.rm=FALSE,...){
-  design$variables<-getvars(x, design$db$connection, design$db$tablename,updates=design$updates)
+  design$variables<-getvars(x, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset)
   NextMethod("svytotal",design)
 }
 
 svyquantile.DBIsvydesign<-function(x, design,quantiles,...){
-  design$variables<-getvars(x, design$db$connection, design$db$tablename,updates=design$updates)
+  design$variables<-getvars(x, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset)
   NextMethod("svyquantile",design)
 }
 
@@ -98,7 +101,7 @@ dropFactor<-function(mf, w){
 }
 
 svyglm.DBIsvydesign<-function(formula, design,...){
-  design$variables<-dropFactor(getvars(formula, design$db$connection, design$db$tablename,updates=design$updates),
+  design$variables<-dropFactor(getvars(formula, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset),
                                weights(design))
   NextMethod("svyglm",design)
 }
@@ -106,7 +109,7 @@ svyglm.DBIsvydesign<-function(formula, design,...){
 
 
 svyplot.DBIsvydesign<-function(formula,design,...){
-  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates)
+  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset)
   design$variables[weights(design)==0,]<-NA
   NextMethod("svyplot",design)
 }
@@ -114,13 +117,13 @@ svyplot.DBIsvydesign<-function(formula,design,...){
 
 svycoplot.DBIsvydesign<-function(formula,design, style=c("hexbin","transparent"),
                             basecol="black",alpha=c(0,0.8),hexscale=c("relative","absolute"),...){
-  design$variables<-getvars(formula, design$db$connection, design$db$tablename, updates=design$updates)
+  design$variables<-getvars(formula, design$db$connection, design$db$tablename, updates=design$updates, subset=design$subset)
   design$variables[weights(design)==0,]<-NA
   NextMethod("svycoplot",design)
 }
 
 svyboxplot.DBIsvydesign<-function(formula,design, ...){
-  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates)
+  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset)
   design$variables[weights(design)==0,]<-NA
   class(design)<-setdiff(class(design),"DBIsvydesign")
   svyboxplot(formula,design,...)
@@ -128,13 +131,13 @@ svyboxplot.DBIsvydesign<-function(formula,design, ...){
 
 
 svycdf.DBIsvydesign<-function(formula,design, na.rm=TRUE, ...){
-  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates)
+  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset)
   NextMethod("svycdf",design)
 
 }
 
 svyolr.DBIsvydesign<-function(formula,design,...){
-  design$variables<-dropFactor(getvars(formula, design$db$connection, design$db$tablename,updates=design$updates),
+  design$variables<-dropFactor(getvars(formula, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset),
                                weights(design))
   NextMethod("svyolr",design)
 }
@@ -146,58 +149,58 @@ svycoxph.DBIsvydesign<-function(formula,design,...){
 }
 
 svyvar.DBIsvydesign<-function(x,design,na.rm=FALSE,...){
-  design$variables<-getvars(x, design$db$connection, design$db$tablename,updates=design$updates)
+  design$variables<-getvars(x, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset)
   NextMethod("svyvar",design)
 }
 
 
 
 svykm.DBIsvydesign<-function(formula,design,...){
-  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates)
+  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset)
   NextMethod("svykm",design)
 }
 
 
 svykappa.DBIsvydesign<-function(formula,design,...){
-  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates)
+  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset)
   NextMethod("svykappa",design)
 }
 
 
 svysmooth.DBIsvydesign<-function(formula,design,method=c("locpoly","quantreg"),bandwidth,quantile,df,...){
-  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates)
+  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset)
   NextMethod("svysmooth",design)
 }
 
 
 svychisq.DBIsvydesign<-function(formula,design,...){
-  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates)
+  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset)
   NextMethod("svychisq",design)
 }
 
 svyratio.DBIsvydesign<-function(numerator, denominator, design,...){
-  design$variables<-cbind(getvars(numerator,design$db$connection, design$db$tablename,updates=design$updates),
-                          getvars(denominator,design$db$connection, design$db$tablename,updates=design$updates))
+  design$variables<-cbind(getvars(numerator,design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset),
+                          getvars(denominator,design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset))
   NextMethod("svyratio",design)
 
 }
 
 
 svyby.DBIsvydesign<-function(formula, by, design,...){
-  design$variables<-cbind(getvars(formula,design$db$connection, design$db$tablename,updates=design$updates),
-                          getvars(by,design$db$connection, design$db$tablename,updates=design$updates))
+  design$variables<-cbind(getvars(formula,design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset),
+                          getvars(by,design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset))
   class(design)<-setdiff(class(design),"DBIsvydesign")
   svyby(formula,by,design,...)
 }
 
 svytable.DBIsvydesign<-function(formula,design,...){
-  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates)
+  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset)
   NextMethod("svytable",design)
 }
 
 
 calibrate.DBIsvydesign<-function(design,formula,...){
-  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates)
+  design$variables<-getvars(formula, design$db$connection, design$db$tablename,updates=design$updates, subset=design$subset)
   NextMethod("calibrate",design)
 }
 postStratify.DBIsvydesign<-function(design, strata, population, partial = FALSE, ...) .NotYetImplemented()
@@ -208,7 +211,7 @@ postStratify.DBIsvydesign<-function(design, strata, population, partial = FALSE,
 subset.DBIsvydesign<-function (x, subset, ...) 
 {
     e <- substitute(subset)
-    x$variables<-getvars(make.formula(all.vars(e)), x$db$connection, x$db$tablename,updates=x$updates)
+    x$variables<-getvars(make.formula(all.vars(e)), x$db$connection, x$db$tablename,updates=x$updates, subset=x$subset)
     r <- eval(e, x$variables, parent.frame())
     r <- r & !is.na(r)
     x <- x[r, ]
@@ -218,6 +221,8 @@ subset.DBIsvydesign<-function (x, subset, ...)
 
 
 
+
+
 dim.DBIsvydesign<-function(x){
   w<-weights(x)
   nrow<-sum(w!=0)
@@ -229,6 +234,20 @@ dim.DBIsvydesign<-function(x){
    c(nrow,ncol)
 }
 
+
+dim.DBIrepdesign<-function(x){
+  if (is.null(x$subset))
+    nrow <-nrow(x$repweights)
+  else
+    nrow<-length(x$subset)
+  coln<-names(dbGetQuery(x$db$conn, paste("select * from", x$db$tablename, "limit 1")))
+  if (!is.null(x$updates)){
+    update.names<-do.call(c, lapply(x$updates, names))
+    ncol<-length(unique(c(coln,update.names)))
+  } else ncol<-length(coln)
+  c(nrow,ncol)
+}
+
 dimnames.DBIsvydesign<-function(x){
    rown<-rownames(x$cluster)[weights(x)!=0]
    coln<-names(dbGetQuery(x$db$conn, paste("select * from", x$db$tablename, "limit 1")))
@@ -240,6 +259,19 @@ dimnames.DBIsvydesign<-function(x){
 }
 
 
+dimnames.DBIrepdesign<-function(x){
+   if (is.null(x$subset))
+     rown<-rownames(x$cluster)
+   else 
+     rown<-rownames(x$cluster)[x$subset]
+   coln<-names(dbGetQuery(x$db$conn, paste("select * from", x$db$tablename, "limit 1")))
+   if (!is.null(x$updates)){
+     update.names<-do.call(c, lapply(x$updates, names))
+     coln<-unique(c(coln,update.names))
+   }
+   list(rown,coln)
+}
+
 "[.DBIsvydesign"<-function (x, i, ..., drop = TRUE) 
 {
   if (!missing(i)) {
@@ -267,3 +299,103 @@ dimnames.DBIsvydesign<-function(x){
   x
 }
 
+
+"[.DBIrepdesign"<-function (x, i, j, drop = FALSE) 
+{
+    if (!missing(i)) {
+        pwt <- x$pweights
+        if (is.data.frame(pwt)) 
+            pwt <- pwt[[1]]
+        x$pweights <- pwt[i]
+        x$repweights <- x$repweights[i, , drop = FALSE]
+        if (!is.null(x$selfrep)) 
+            x$selfrep <- x$selfrep[i]
+        if (is.null(x$subset))
+          x$subset<-(1:nrow(x$variables))[i]
+        else
+          x$subset<-x$subset[i]
+        if (!missing(j)) 
+            x$variables <- x$variables[i, j, drop = FALSE]
+        else x$variables <- x$variables[i, , drop = FALSE]
+        x$degf <- NULL
+        x$degf <- degf(x)
+    }
+    else {
+        x$variables <- x$variables[, j, drop = FALSE]
+    }
+    x
+}
+
+
+svrepdesign.character<-function (variables=NULL,repweights=NULL, weights=NULL,
+                                 data=NULL,type=c("BRR","Fay","JK1", "JKn","bootstrap","other"),
+                                 combined.weights=TRUE, rho=NULL, bootstrap.average=NULL,
+                                 scale=NULL,rscales=NULL,fpc=NULL, fpctype=c("fraction","correction"),
+                                 dbtype="SQLite", dbname,
+                                 ...) 
+{
+
+  if (dbtype == "ODBC"){
+    library(RODBC)
+    if (dbname=="")
+      dbconn<-odbcDriverConnect(dbname,...)
+    else
+      dbconn<-odbcConnect(dbname,...)
+  } else {
+    db<-dbDriver(dbtype)
+    dbconn<- dbConnect(db, dbname,...)
+  }
+  if (is.character(repweights)){
+    allvars<-names(dbGetQuery(dbconn, paste("select * from",data,"limit 1"))) 
+    design.vars<-c(all.vars(weights),grep(repweights,allvars,value=TRUE))
+  } else {
+    design.vars<-c(all.vars(weights),all.vars(repweights))
+  }
+
+  
+  design.query<-paste("select", paste(design.vars,collapse=","), "from", data)
+  if (dbtype=="ODBC")
+    design.data<-sqlQuery(dbconn, design.query)
+  else
+    design.data<-dbGetQuery(dbconn, design.query)
+    
+  rval<-svrepdesign(variables=variables,repweights=repweights, weights=weights, type=type,
+                    data=design.data,
+                    combined.weights=TRUE, rho=rho, bootstrap.average=NULL,
+                    scale=scale,rscales=rscales,fpc=fpc, fpctype=c("fraction","correction"))
+  
+  rval$db<-list(dbname=dbname, tablename=data, connection=dbconn, dbtype=dbtype)
+  rval$variables<-NULL
+  rval$call<-sys.call(-1)
+  if (dbtype=="ODBC")
+    class(rval)<-c("ODBCrepdesign","ODBCsvydesign",class(rval))
+  else
+    class(rval)<-c("DBIrepdesign","DBIsvydesign",class(rval))
+  rval
+}
+
+print.DBIrepdesign<-function(x,...){
+  cat("DB-backed replicate weight design\n")
+  print.svyrep.design(x,...)
+  if (!checkConnection(x$db$connection, error=FALSE))
+    cat("<DBI Connection closed>\n")
+  invisible(x)
+}
+
+print.ODBCrepdesign<-function(x,...){
+  cat("ODBC-backed replicate weight design\n")
+  print.svyrep.design(x,...)
+   if (!checkConnection(x$db$connection, error=FALSE))
+    cat("<ODBC Connection closed>\n")
+  invisible(x)
+}
+
+summary.DBIrepdesign<-function(object,...){
+   summary.svyrep.design(object,...)
+}
+
+summary.ODBCrepdesign<-function(object,...){
+   summary.svyrep.design(object,...)
+}
+
+
