@@ -18,7 +18,7 @@ regTermTest<-function(model, test.terms, null=NULL, df=Inf, method=c("Wald","LRT
   okbeta<-!is.na(coef(model,na.rm=FALSE)) ## na.rm for svyglm
   tt<-attr(terms(model),"term.labels")
   aa<-attr(model.matrix(model),"assign")[okbeta]
-  if((inherits(model,"coxph")|| inherits(model,"svyloglin")) && attr(terms(model),"intercept"))
+  if((inherits(model,"coxph")|| inherits(model,"svyloglin") || inherits(model,"svyolr"))  && attr(terms(model),"intercept"))
     aa<-aa[-1]
   index<-which(aa %in% match(canonicalOrder(test.terms),canonicalOrder(tt)))
   if (any(is.na(index)))
